﻿Imports System.IO
Imports System.Security.Cryptography
Imports System.Text

Public Class FestivalCryptoService

    Public Structure KeyPair
        Dim PrivateKey As String
        Dim PublicKey As String
    End Structure

    Public Structure AESData
        Dim IVb64 As String
        Dim Keyb64 As String
    End Structure

#Region "AES"

    ''' <summary>
    ''' Generiert einen zufälligen AES-IV und einen AES-Key und gibt ein AESData-Objekt mit den Base64-Strings der Werte zurück.
    ''' </summary>
    ''' <returns></returns>
    Public Function GetAESData() As FestivalCryptoService.AESData
        Dim tReturn As New AESData
        Dim AESCSP As New AesCryptoServiceProvider
        AESCSP.GenerateIV()
        AESCSP.GenerateKey()
        tReturn.IVb64 = Convert.ToBase64String(AESCSP.IV)
        tReturn.Keyb64 = Convert.ToBase64String(AESCSP.Key)
        AESCSP.Dispose()
        Return tReturn
    End Function

    ''' <summary>
    ''' Verschlüsselt einen String mit AES.
    ''' </summary>
    ''' <param name="tEncrypt">Zu verschlüsselnder String</param>
    ''' <param name="IV">Base64-String des IV</param>
    ''' <param name="Key">Base64-String des Keys</param>
    ''' <returns></returns>
    Public Function AESEncrypt(ByVal tEncrypt As String, ByVal IV As String, ByVal Key As String) As String
        Dim AESCSP As New AesCryptoServiceProvider
        Dim AESCT As ICryptoTransform = AESCSP.CreateEncryptor(Convert.FromBase64String(Key), Convert.FromBase64String(IV))
        Dim tReturnBytes() As Byte
        Dim MS As New MemoryStream
        Using CS As New CryptoStream(MS, AESCT, CryptoStreamMode.Write)
            Using SW As New StreamWriter(CS)
                SW.Write(tEncrypt)
            End Using
        End Using
        tReturnBytes = MS.ToArray
        AESCSP.Dispose()
        Return Convert.ToBase64String(tReturnBytes)
    End Function

    ''' <summary>
    ''' Entschlüsselt einen String mit AES.
    ''' </summary>
    ''' <param name="tDecrypt">Zu entschlüsselnder String</param>
    ''' <param name="IV">Base64-String des IV</param>
    ''' <param name="Key">Base64-String des Keys</param>
    ''' <returns></returns>
    Public Function AESDecrypt(ByVal tDecrypt As String, ByVal IV As String, ByVal Key As String) As String
        Dim AESCSP As New AesCryptoServiceProvider
        Dim AESCT As ICryptoTransform = AESCSP.CreateDecryptor(Convert.FromBase64String(Key), Convert.FromBase64String(IV))
        Dim MS As New MemoryStream(Convert.FromBase64String(tDecrypt))
        Dim CS As New CryptoStream(MS, AESCT, CryptoStreamMode.Read)
        Dim SR As New StreamReader(CS)
        Dim tReturn As String = SR.ReadToEnd
        Return tReturn
    End Function

#End Region

#Region "RSA"

    ''' <summary>
    ''' Generiert ein zufälliges RSA-Schlüsselpaar und gibt ein KeyPair-Objekt mit den XML-Strings der Schlüssel zurück.
    ''' </summary>
    ''' <returns></returns>
    Public Function GetRSAKeys() As FestivalCryptoService.KeyPair
        Dim tReturn As New KeyPair
        Dim RSACSP As New RSACryptoServiceProvider(2048)
        tReturn.PrivateKey = RSACSP.ToXmlString(True)
        tReturn.PublicKey = RSACSP.ToXmlString(False)
        RSACSP.Dispose()
        Return tReturn
    End Function

    ''' <summary>
    ''' Verschlüsselt einen String mit RSA.
    ''' </summary>
    ''' <param name="tEncrypt">Zu verschlüsselnder String</param>
    ''' <param name="PublicKey">XML-String des PublicKeys</param>
    ''' <returns></returns>
    Public Function RSAEncrypt(ByVal tEncrypt As String, ByVal PublicKey As String) As String
        Dim RSACSP As New RSACryptoServiceProvider(2048)
        Dim tEncryptBytes() As Byte = Encoding.UTF8.GetBytes(tEncrypt)
        Dim tReturn As String
        RSACSP.FromXmlString(PublicKey)
        tReturn = Convert.ToBase64String(RSACSP.Encrypt(tEncryptBytes, True))
        RSACSP.Dispose()
        Return tReturn
    End Function

    ''' <summary>
    ''' Entschlüsselt einen String mit RSA.
    ''' </summary>
    ''' <param name="tDecrypt">Zu entschlüsselnder String</param>
    ''' <param name="PrivateKey">XML-String des PrivateKeys</param>
    ''' <returns></returns>
    Public Function RSADecrypt(ByVal tDecrypt As String, ByVal PrivateKey As String) As String
        Dim RSACSP As New RSACryptoServiceProvider(2048)
        Dim tDecryptBytes() As Byte = Convert.FromBase64String(tDecrypt)
        Dim tReturn As String
        RSACSP.FromXmlString(PrivateKey)
        tReturn = Encoding.UTF8.GetString(RSACSP.Decrypt(tDecryptBytes, True))
        RSACSP.Dispose()
        Return tReturn
    End Function

#End Region

#Region "SHA"

    ''' <summary>
    ''' Errechnet den SHA256-Hashwert eines Strings.
    ''' </summary>
    ''' <param name="tHash">Zu hashender String</param>
    ''' <returns></returns>
    Public Function SHA256ComputeHash(ByVal tHash As String) As String
        Dim SHACSP As New SHA256CryptoServiceProvider
        Dim tHashBytes() As Byte = Encoding.UTF8.GetBytes(tHash)
        Dim tReturn As String
        tReturn = Convert.ToBase64String(SHACSP.ComputeHash(tHashBytes))
        SHACSP.Dispose()
        Return tReturn
    End Function

#End Region

End Class
